module ast.Module;

import sema.Scope,
       sema.VC,
       sema.Symbol;

import ast.Decl;

class Module
{
    this(char[] moduleName)
    {
        this.moduleName = moduleName;
    }

    void addDecl(Decl decl)
    {
        if(!decl)
            return;
        switch(decl.declType)
        {
            case DeclType.FuncDecl:
                functions ~= cast(FuncDecl)decl;
                break;
            case DeclType.VarDecl:
                vars ~= cast(VarDecl)decl;
                break;
            case DeclType.StructDecl:
                structs ~= cast(StructDecl)decl;
                break;
            case DeclType.ClassDecl:
                classes ~= cast(ClassDecl)decl;
                break;
            case DeclType.InterfaceDecl:
                interfaces ~= cast(InterfaceDecl)decl;
                break;
            case DeclType.AliasDecl:
                aliases ~= cast(AliasDecl)decl;
                break;
            default:
                assert(0, "DeclType not implemented");
        }
        decls ~= decl;
    }

    void verify(VC vc)
    {
        foreach (decl; decls)
            decl.verify(vc);
    }

    VarDecl[]       vars;
    FuncDecl[]      functions;
    StructDecl[]    structs;
    ClassDecl[]     classes;
    InterfaceDecl[] interfaces;
    AliasDecl[]     aliases;
    Decl[] decls;

    char[] moduleName;
    Scope env;
    Symbol symbol;
    bool outputModule = true;
}

